/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LogicalExpressionVO.as 4869 2009-08-11 11:14:12Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual
{

import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.AbstractRulePackageMemberVO;

/**
* A logical expression
*
* @author Julien Nicoulaud
*/
[RemoteClass(alias="org.ow2.jasmine.rules.manager.model.visual.LogicalExpression")]
public class LogicalExpressionVO extends AbstractRulePackageMemberVO
                                 implements ILogicalExpressionVO
{
	
	// =========================================================================
	// Properties
	// =========================================================================
    
    /**
    * The displayable name of the element type
    */
    public static const RULE_ELEMENT_TYPE_NAME : String = "Logical expression";
    
    /**
    * The description of the element type
    */
    public static const RULE_ELEMENT_TYPE_DESCRIPTION : String =
        "Insert a logical expression to test several conditions.";
        
    /**
    * The supported logical operators
    */
    public static var LOGICAL_OPERATORS : ArrayCollection
        = new ArrayCollection(['OR','AND']);
    
    /**
    * The operator
    */
    private var _operator : String = "";
    
    /**
    * The operands
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.ILHSElementVO')]
    private var _operands : ArrayCollection;

    /**
    * The imports
    */
    [ArrayElementType('String')]
    private var _imports : ArrayCollection;

    /**
    * The variables defined in the rule
    */
    [ArrayElementType('String')]
    private var _variables : ArrayCollection;
    
    
    // =========================================================================
    // Getters and setters
    // =========================================================================
    
    /**
    * Set the operator
    */
    public function set operator(value:String):void
    {
        // Store the old value
        var oldValue : Object = _operator;
        
        // Assign the value
        _operator = value;

        // Dispatch a property change event
        notifyPropertyChange("operator",oldValue,value);
    }
    
    /**
    * Get the operator
    */
    [Bindable]
    public function get operator():String
    {
        return _operator;
    }
    
    /**
    * Set the operands
    */
    public function set operands(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _operands;

        // Assign the value
        _operands = value;
        
        // Propagate the change to the children
        for each (var lhsElement : ILHSElementVO in operands) {
            lhsElement.imports = imports;
            lhsElement.variables = variables;
        }
                
        // Listen to the operands collection changes
        _operands.addEventListener(CollectionEvent.COLLECTION_CHANGE,operandsCollectionChangeHandler);
                
        // Dispatch a property change event
        notifyPropertyChange("operands",oldValue,value);
    }
    
    /**
    * Get the operands
    */
    [Bindable]
    public function get operands():ArrayCollection
    {
        // If the field is not initialized, do it
        if (_operands == null) {
            _operands = new ArrayCollection();
            _operands.addEventListener(CollectionEvent.COLLECTION_CHANGE,operandsCollectionChangeHandler,false,0,true);
        }
        return _operands;
    }
    
    /**
    * Set the imports collection
    */    
    public function set imports(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _imports;
        
        // Assign the value
        
        _imports = value;
        
        // Propagate the change to the children
        for each (var operand : ILHSElementVO in operands) {
            operand.imports = imports;
        }
        
        // Dispatch a property change event
        notifyPropertyChange("imports",oldValue,value);
    }
    
    /**
    * Get the imports collection
    */
    [Transient]
    [Bindable]
    public function get imports():ArrayCollection
    {
        // Initialize the property if needed
        if (_imports == null) {
            _imports = new ArrayCollection();
        }
        return _imports;
    }
    
    /**
    * Set the variables collection
    */    
    public function set variables(value:ArrayCollection):void
    {
        // Store the old value
        var oldValue : Object = _variables;
        
        // Assign the value
        _variables = value;
        
        // Propagate the change to the children
        for each (var operand : ILHSElementVO in operands) {
            operand.variables = variables;
        }
        
        // Dispatch a property change event
        notifyPropertyChange("variables",oldValue,value);
    }
    
    /**
    * Get the variables collection
    */
    [Transient]
    [Bindable]
    public function get variables():ArrayCollection
    {
        // Initialize the property if needed
        if (_variables == null) {
            _variables = new ArrayCollection();
        }
        return _variables;
    }
    
    
    // =========================================================================
    // Implementation of IRuleElementVO
    // =========================================================================
    
    /**
    * The displayable name of the element type
    */    
    public function getName():String
    {
        return RULE_ELEMENT_TYPE_NAME;
    }
        
    /**
    * The description of the element type
    */
    public function getDescription():String
    {
        return RULE_ELEMENT_TYPE_DESCRIPTION;
    }
    
    
    // =========================================================================
    // Implementation of ILogicalExpressionVO
    // =========================================================================
    
    /**
    * Get the generated source in Drools syntax
    */
    override public function getDrlSource():String
    {
    	var source : String = "(";
    	
        // Format conditions
        var first:Boolean = true;
        for each (var operand:ILHSElementVO in operands) {
            if (first) {
                first = false;
            } else {
                source += ' ' + operator + ' ';
            }
            source += operand.getDrlSource();
        }
        source += ")";
        
        return source;
    }
    
    
    // =========================================================================
    // Events handlers
    // =========================================================================
    
    /**
    * When the operands list changes
    */
    private function operandsCollectionChangeHandler(event:CollectionEvent):void
    {
        // Bind some synchronized variables
        switch (event.kind) {
            case CollectionEventKind.ADD:
                for each (var operand : ILHSElementVO in event.items) {
                    operand.imports = imports;
                    operand.variables = variables;
                }
                break;
        }
        
        // Notify the change        
        notifyPropertyChange("operands",_operands,operands);
    }
    
}
}
